package cn.newfeifan.mall.module.distri.controller.admin.integraltransferlog;

import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.*;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.pojo.CommonResult;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;
import static cn.newfeifan.mall.framework.common.pojo.CommonResult.success;
import cn.newfeifan.mall.module.distri.controller.admin.integraltransferlog.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.integraltransferlog.IntegralTransferLogDO;
import cn.newfeifan.mall.module.distri.service.integraltransferlog.IntegralTransferLogService;

@Tag(name = "管理后台 - 消费分转让记录")
@RestController
@RequestMapping("/distri/consumption-transfer-log")
@Validated
public class IntegralTransferLogController {

    @Resource
    private IntegralTransferLogService integralTransferLogService;

    @GetMapping("/get")
    @Operation(summary = "获得消费分转让记录")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('distri:consumption-transfer-log:query')")
    public CommonResult<IntegralTransferLogRespVO> getConsumptionTransferLog(@RequestParam("id") Long id) {
        IntegralTransferLogDO consumptionTransferLog = integralTransferLogService.getConsumptionTransferLog(id);
        return success(BeanUtils.toBean(consumptionTransferLog, IntegralTransferLogRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得消费分转让记录分页")
    @PreAuthorize("@ss.hasPermission('distri:consumption-transfer-log:query')")
    public CommonResult<PageResult<IntegralTransferLogRespVO>> getConsumptionTransferLogPage(@Valid IntegralTransferLogPageReqVO pageReqVO) {
        PageResult<IntegralTransferLogDO> pageResult = integralTransferLogService.getConsumptionTransferLogPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, IntegralTransferLogRespVO.class));
    }

}