package cn.newfeifan.mall.module.distri.service.integraltopuplog;

import cn.newfeifan.mall.framework.common.exception.ErrorCode;
import cn.newfeifan.mall.module.distri.constant.DistriConstants;
import cn.newfeifan.mall.module.distri.controller.admin.duser.vo.DuserSaveReqVO;
import cn.newfeifan.mall.module.distri.controller.admin.integralchangelog.vo.IntegralChangeLogSaveReqVO;
import cn.newfeifan.mall.module.distri.controller.admin.integral.vo.IntegralSaveReqVO;
import cn.newfeifan.mall.module.distri.dal.dataobject.duser.DuserDO;
import cn.newfeifan.mall.module.distri.dal.dataobject.integral.IntegralDO;
import cn.newfeifan.mall.module.distri.dal.dataobject.orderpercentage.OrderPercentageDO;
import cn.newfeifan.mall.module.distri.enums.IntegralEnum;
import cn.newfeifan.mall.module.distri.enums.IntegralTypeEnum;
import cn.newfeifan.mall.module.distri.service.duser.DuserService;
import cn.newfeifan.mall.module.distri.service.integralchangelog.IntegralChangeLogService;
import cn.newfeifan.mall.module.distri.service.integral.IntegralService;
import cn.newfeifan.mall.module.distri.service.orderpercentage.OrderPercentageService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import cn.newfeifan.mall.module.distri.controller.admin.integraltopuplog.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.integraltopuplog.IntegralTopUpLogDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;

import cn.newfeifan.mall.module.distri.dal.mysql.consumptiontopuplog.ConsumptionTopUpLogMapper;

import java.util.List;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;

/**
 * 平台消费分充值记录 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class IntegralTopUpLogServiceImpl implements IntegralTopUpLogService {

    @Resource
    private ConsumptionTopUpLogMapper consumptionTopUpLogMapper;

    @Resource
    private OrderPercentageService orderPercentageService;

    @Resource
    private IntegralService integralService;

    @Resource
    private IntegralChangeLogService integralChangeLogService;
    @Resource
    private DuserService duserService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createConsumptionTopUpLog(IntegralTopUpLogSaveReqVO createReqVO) {
        // 充值的额度
        Long agentQuota = createReqVO.getIntegralPoints();
        OrderPercentageDO orderPercentageDO = orderPercentageService.queryStatus();
        // 充值额度校验
        quotaVerify(createReqVO.getUserId(), createReqVO.getIntegralPoints(), orderPercentageDO);

        // 额度放大百万倍,存储需要(方便换算),和积分一样
        createReqVO.setIntegralPoints(createReqVO.getIntegralPoints() * DistriConstants.MILLION);

        // 获取用户的会员类别信息
        DuserDO user = duserService.getDuserByUser(createReqVO.getUserId());

        // 充值倍率
        if (createReqVO.getIntegralPoints() >= Long.parseLong(orderPercentageDO.getTriggerMagnificationPoints()) * DistriConstants.MILLION) {
            createReqVO.setPracticalIntegralPoints((long) (createReqVO.getIntegralPoints() * Double.parseDouble(orderPercentageDO.getIntegralMagnification())));
        } else {
            createReqVO.setPracticalIntegralPoints(createReqVO.getIntegralPoints());
        }

        // 插入
        IntegralTopUpLogDO consumptionTopUpLog = BeanUtils.toBean(createReqVO, IntegralTopUpLogDO.class);
        consumptionTopUpLogMapper.insert(consumptionTopUpLog);

        // 修改用户钱包
        IntegralDO integralDO = integralService.selectByUser(consumptionTopUpLog.getUserId());
        integralDO.setRedCurrentQuota(integralDO.getRedCurrentQuota() + consumptionTopUpLog.getPracticalIntegralPoints());
        integralService.updateIntegral(BeanUtils.toBean(integralDO, IntegralSaveReqVO.class));

        // 记录用户消费分变动日志
        integralChangeLogService.createConsumptionChangeLog(IntegralChangeLogSaveReqVO.builder()
                .userId(consumptionTopUpLog.getUserId())
                .integralPoints(consumptionTopUpLog.getIntegralPoints())
                .afterIntegralPoints(integralDO.getRedCurrentQuota())
                .integralStatus(IntegralEnum.PT_TOP_TUP.getType())
                .integralType(IntegralTypeEnum.RED_INTEGRAL.getType())
                .practicalIntegralPoints(consumptionTopUpLog.getPracticalIntegralPoints())
                .generateUserId(getLoginUserId())
                .build());

        // 发送短信通知用户充值成功
//        MemberUserDO user = memberUserService.getUser(consumptionTopUpLog.getUserId());
//        smsCodeService.sendSmsCode(SmsCodeSendReqDTO.builder()
//                .mobile(user.getMobile())
//                .scene(SmsSceneEnum.CONSUMPTION_POINTS_TOP_UP.getScene())
//                .createIp(getClientIP())
//                .build());


        // 当单笔充值大于设定金额时,将用户设置为代理商
        if(agentQuota >= Long.parseLong(orderPercentageDO.getTriggerMagnificationPoints()) && user.getCategoryId() != 3){
            user.setCategoryId(3);
            duserService.updateDuser(BeanUtils.toBean(user, DuserSaveReqVO.class));
        }

        // 返回
        return consumptionTopUpLog.getId();
    }

    /**
     * 校验充值额度
     *
     * @param userId            充值用户
     * @param consumptionPoints 额度
     * @param orderPercentageDO 设定额度
     */
    private void quotaVerify(Long userId, Long consumptionPoints, OrderPercentageDO orderPercentageDO) {
        // 金额不小于1就行
        if (consumptionPoints < 1) {
            throw exception(new ErrorCode(1_002_030_035, "充值额度不能小于1"));
        }

//        // 充值额度校验
//        Long quota = Long.parseLong(orderPercentageDO.getFollowUpConsumption());      // 后续充值
//        if (consumptionTopUpLogMapper.selectList(ConsumptionTopUpLogDO::getUserId, userId).isEmpty()) {
//            // 首次充值
//            quota = Long.parseLong(orderPercentageDO.getFirstTopUpConsumption());
//        }
//        if (consumptionPoints < quota) {
//            ErrorCode ERROR = new ErrorCode(1_002_030_035, "充值的额度不能小于规定额度");
//            throw exception(ERROR);
//        }
    }

    @Override
    public void updateConsumptionTopUpLog(IntegralTopUpLogSaveReqVO updateReqVO) {
        // 校验存在
        validateConsumptionTopUpLogExists(updateReqVO.getId());
        // 更新
        IntegralTopUpLogDO updateObj = BeanUtils.toBean(updateReqVO, IntegralTopUpLogDO.class);
        consumptionTopUpLogMapper.updateById(updateObj);
    }

    private void validateConsumptionTopUpLogExists(Long id) {
        if (consumptionTopUpLogMapper.selectById(id) == null) {
            throw exception(CONSUMPTION_TOP_UP_LOG_NOT_EXISTS);
        }
    }

    @Override
    public IntegralTopUpLogDO getConsumptionTopUpLog(Long id) {
        return consumptionTopUpLogMapper.selectById(id);
    }

    @Override
    public PageResult<IntegralTopUpLogDO> getConsumptionTopUpLogPage(IntegralTopUpLogPageReqVO pageReqVO) {
        return consumptionTopUpLogMapper.selectPage(pageReqVO);
    }

    @Override
    public IntegralInfoVO isFirstConsumption(Long userId) {
        List<IntegralTopUpLogDO> integralTopUpLogDOS = consumptionTopUpLogMapper.selectList(IntegralTopUpLogDO::getUserId, userId);

        IntegralInfoVO build = IntegralInfoVO.builder().build();
        OrderPercentageDO orderPercentageDO = orderPercentageService.queryStatus();
//        if (consumptionTopUpLogDOS.isEmpty()) {
//            build.setIsFirstConsumption(true);
//            build.setMinimumConsumptionPoints(Long.valueOf(orderPercentageDO.getFirstTopUpConsumption()));
//        } else {
//            build.setIsFirstConsumption(false);
//            build.setMinimumConsumptionPoints(Long.valueOf(orderPercentageDO.getFollowUpConsumption()));
//        }
        build.setConsumptionMagnification(orderPercentageDO.getIntegralMagnification());
        build.setTriggerMagnificationPoints(orderPercentageDO.getTriggerMagnificationPoints());
        return build;
    }

    @Override
    public List<IntegralTopUpLogDO> selectList() {
        return consumptionTopUpLogMapper.selectList();
    }

    @Override
    public PtTopUpWithUserTopUpRespVO ptTopUpWithUserTopUpList() {
        return PtTopUpWithUserTopUpRespVO.builder()
                .topUpList(consumptionTopUpLogMapper.ptTopUpWithUserTopUpList())
                .totalConsumptionPoints(consumptionTopUpLogMapper.getTotalConsumptionPoints())
                .build();
    }


}